from django.views.generic.list import ListView
from django.conf import settings

from .models import DjangoActionLogEntry


class Log(ListView):
    model = DjangoActionLogEntry

    def get_queryset(self, *args, **kwargs):
        queryset = super().get_queryset(*args, **kwargs)
        if not self.request.user.is_staff:
            queryset = queryset.filter(user=self.request.user)
            if app_labels := getattr(
                settings, "LOG_LIST_NOSTAFF_EXCLUDE_APP_LABELS", None
            ):
                queryset = queryset.exclude(content_type__app_label__in=app_labels)
        elif app_labels := getattr(settings, "LOG_LIST_EXCLUDE_APP_LABELS", None):
            queryset = queryset.exclude(content_type__app_label__in=app_labels)
        return queryset



from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models


class DjangoActionLogEntryManager(models.Manager):
    def log_action(self, user, obj=None, message="", metadata={}):
        username = "System action"
        if user:
            username = str(user)
        entry = self.model.objects.create(user=username)
        if obj:
            content_type = ContentType.objects.get_for_model(obj)
            entry.content_type = content_type
            natural_key = f"{content_type.app_label}.{content_type.name}"
            metadata["natural_key"] = natural_key
            message += f" {natural_key}: {obj}"
            metadata["pk"] = obj.pk
            if hasattr(obj, "id"):
                entry.object_id = obj.id
        if message:
            entry.message = message
        if metadata:
            entry.metadata = metadata
        entry.save()

    def add_something(self, user, obj):
        message = f"{user} added"
        metadata = {"type": "add"}
        self.log_action(user, obj, message, metadata)

    def delete_something(self, user, obj):
        message = f"{user} deleted"
        metadata = {"type": "del"}
        self.log_action(user, obj, message, metadata)

    def change_something(self, user, obj, update_fields=[]):
        message = f"{user} changed"
        metadata = {"type": "change", "update_fields": update_fields}
        self.log_action(user, obj, message, metadata)


class DjangoActionLogEntry(models.Model):
    """
    A custom LogEntry model. Django has its own contrib.admin.LogEntry, but that
    is primarily used for actions in the admin area. Using a custom LogEntry we
    are more flexible.
    """

    user = models.CharField(max_length=150, editable=False)
    action_time = models.DateTimeField(auto_now_add=True, editable=False)

    content_type = models.ForeignKey(
        ContentType, on_delete=models.CASCADE, blank=True, null=True
    )
    object_id = models.PositiveIntegerField(blank=True, null=True)
    content_object = GenericForeignKey("content_type", "object_id")

    message = models.TextField(null=True)
    metadata = models.JSONField(null=True)

    objects = DjangoActionLogEntryManager()

    class Meta:
        ordering = ["-action_time"]



# Generated by Django 4.2.10 on 2024-02-16 15:03

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
    ]

    operations = [
        migrations.CreateModel(
            name="DjangoActionLogEntry",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("user", models.CharField(editable=False, max_length=150)),
                ("action_time", models.DateTimeField(auto_now_add=True)),
                ("object_id", models.PositiveIntegerField(blank=True, null=True)),
                ("message", models.TextField(null=True)),
                ("metadata", models.JSONField(null=True)),
                (
                    "content_type",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="contenttypes.contenttype",
                    ),
                ),
            ],
            options={
                "ordering": ["-action_time"],
            },
        ),
    ]



from django import template

register = template.Library()


@register.filter
def template_names(obj):
    template_names = ["django_action_log/logentry.html"]
    if obj.content_type:
        template_names.append(
            f"django_action_log/{obj.content_type.app_label}_logentry.html"
        )
        template_names.append(
            f"django_action_log/{obj.content_type.app_label}.{obj.content_type.model}_logentry.html"
        )
    return template_names



from django.db.models.signals import post_save, post_delete
from django.dispatch import receiver
from crum import get_current_request

from .models import DjangoActionLogEntry


@receiver(post_save)
def log_save(sender, instance, created, raw, using, update_fields, **kwargs):
    if sender is DjangoActionLogEntry:
        return
    user = None
    request = get_current_request()
    if request:
        user = request.user
    if created:
        DjangoActionLogEntry.objects.add_something(user, instance)
    else:
        if update_fields:
            update_fields = list(update_fields)
        DjangoActionLogEntry.objects.change_something(user, instance, update_fields)


@receiver(post_delete)
def log_delete(sender, instance, using, origin, **kwargs):
    if sender is DjangoActionLogEntry:
        return
    user = None
    request = get_current_request()
    if request:
        user = request.user
    DjangoActionLogEntry.objects.delete_something(user, instance)



from django.apps import AppConfig


class DjangoActionLogLogConfig(AppConfig):
    default_auto_field = "django.db.models.AutoField"
    name = "django_action_logger"

    def ready(self):
        from . import signals
